"use strict";
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var bridge_1 = require('../core/bridge');
var query_1 = require('../core/query');
var pager_1 = require('./pager');
exports.Pager = pager_1.Pager;
var EventEmitter = require('eventemitter3');
var filterObject = require('filter-object');
var deepEqual = require('deep-equal');
var Events;
(function (Events) {
    Events.SEARCH = 'search';
    Events.RESULTS = 'results';
    Events.REFINEMENTS_CHANGED = 'refinements_changed';
    Events.PAGE_CHANGED = 'page_changed';
    Events.REQUEST_CHANGED = 'request_changed';
    Events.RESET = 'reset';
    Events.REWRITE_QUERY = 'rewrite_query';
    Events.SORT = 'sort';
    Events.DETAILS = 'details';
    Events.REDIRECT = 'redirect';
})(Events = exports.Events || (exports.Events = {}));
var FluxCapacitor = (function (_super) {
    __extends(FluxCapacitor, _super);
    function FluxCapacitor(endpoint, config, mask) {
        if (config === void 0) { config = {}; }
        _super.call(this);
        this.originalQuery = '';
        this.originalRefinements = [];
        this.bridge = new bridge_1.BrowserBridge(endpoint, config.https);
        if (config.headers)
            this.bridge.headers = config.headers;
        this.query = new query_1.Query().withConfiguration(filterObject(config, ['*', '!{headers,https}']), mask);
    }
    Object.defineProperty(FluxCapacitor.prototype, "page", {
        get: function () {
            return new pager_1.Pager(this);
        },
        enumerable: true,
        configurable: true
    });
    FluxCapacitor.prototype.search = function (originalQuery) {
        var _this = this;
        if (originalQuery === void 0) { originalQuery = this.originalQuery; }
        this.query.withQuery(originalQuery);
        this.emit(Events.SEARCH, this.query.raw);
        return this.bridge.search(this.query)
            .then(function (results) {
            if (results.redirect)
                _this.emit(Events.REDIRECT, results.redirect);
            var originalRefinements = _this.query.raw.refinements;
            _this.testForChange(originalQuery, originalRefinements);
            Object.assign(_this, { results: results, originalQuery: originalQuery, originalRefinements: originalRefinements });
            _this.emit(Events.RESULTS, results);
            return results;
        });
    };
    FluxCapacitor.prototype.rewrite = function (query, config) {
        var _this = this;
        if (config === void 0) { config = {}; }
        var search;
        if (config.skipSearch) {
            search = Promise.resolve(this.query.withQuery(this.originalQuery = query));
        }
        else {
            search = this.search(query);
        }
        return search.then(function () { return _this.emit(Events.REWRITE_QUERY, query); })
            .then(function () { return query; });
    };
    FluxCapacitor.prototype.resetRecall = function () {
        this.query = new query_1.Query().withConfiguration(this.filteredRequest);
    };
    FluxCapacitor.prototype.reset = function (query) {
        var _this = this;
        if (query === void 0) { query = this.originalQuery; }
        this.resetRecall();
        this.emit(Events.PAGE_CHANGED, { pageIndex: 0 });
        return this.search(query)
            .then(function (res) { return _this.emit(Events.RESET, res); })
            .then(function () { return query; });
    };
    FluxCapacitor.prototype.resize = function (pageSize, offset) {
        this.query.withConfiguration({ pageSize: pageSize });
        if (offset !== undefined) {
            this.query.skip(offset);
            this.emit(Events.PAGE_CHANGED, { pageIndex: this.page.pageFromOffset(offset) });
        }
        return this.search()
            .then(function () { return pageSize; });
    };
    FluxCapacitor.prototype.sort = function (sort, clearSorts) {
        if (clearSorts === void 0) { clearSorts = [sort]; }
        (_a = this.query).withoutSorts.apply(_a, clearSorts).withSorts(sort);
        return this.page.reset();
        var _a;
    };
    FluxCapacitor.prototype.refine = function (refinement, config) {
        if (config === void 0) { config = { reset: true }; }
        this.query.withSelectedRefinements(refinement);
        if (config.skipSearch)
            return Promise.resolve(this.navigationInfo);
        return this.doRefinement(config);
    };
    FluxCapacitor.prototype.unrefine = function (refinement, config) {
        if (config === void 0) { config = { reset: true }; }
        this.query.withoutSelectedRefinements(refinement);
        if (config.skipSearch)
            return Promise.resolve(this.navigationInfo);
        return this.doRefinement(config);
    };
    FluxCapacitor.prototype.details = function (id) {
        var _this = this;
        return this.bridge.search(new query_1.Query()
            .withConfiguration(filterObject(this.query.raw, '{area,collection,language,fields}'))
            .withSelectedRefinements({ navigationName: 'id', type: 'Value', value: id }))
            .then(function (res) {
            if (res.records.length)
                _this.emit(Events.DETAILS, res.records[0]);
            return res;
        });
    };
    FluxCapacitor.prototype.switchCollection = function (collection) {
        this.query.withConfiguration({ collection: collection });
        return this.search();
    };
    FluxCapacitor.prototype.testForChange = function (query, refinements) {
        if (query !== this.originalQuery || !deepEqual(refinements, this.originalRefinements)) {
            this.emit(Events.REQUEST_CHANGED);
        }
    };
    Object.defineProperty(FluxCapacitor.prototype, "filteredRequest", {
        get: function () {
            return filterObject(this.query.raw, '!{query,refinements,skip}');
        },
        enumerable: true,
        configurable: true
    });
    FluxCapacitor.prototype.resetPaging = function (reset) {
        return reset ? this.page.reset() : this.search();
    };
    FluxCapacitor.prototype.doRefinement = function (_a) {
        var _this = this;
        var reset = _a.reset;
        return this.resetPaging(reset)
            .then(function () { return _this.emit(Events.REFINEMENTS_CHANGED, _this.navigationInfo); })
            .then(function () { return _this.navigationInfo; });
    };
    Object.defineProperty(FluxCapacitor.prototype, "navigationInfo", {
        get: function () {
            return {
                available: this.results.availableNavigation,
                selected: this.results.selectedNavigation
            };
        },
        enumerable: true,
        configurable: true
    });
    return FluxCapacitor;
}(EventEmitter));
exports.FluxCapacitor = FluxCapacitor;
