"use strict";
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var query_1 = require('./query');
var axios = require('axios');
var SEARCH = '/search';
var REFINEMENTS = '/refinements';
var REFINEMENT_SEARCH = '/refinement';
var CLUSTER = '/cluster';
var AbstractBridge = (function () {
    function AbstractBridge() {
        this.headers = {};
    }
    AbstractBridge.prototype.search = function (query, callback) {
        if (callback === void 0) { callback = undefined; }
        var _a = this.extractRequest(query), request = _a[0], queryParams = _a[1];
        if (request === null)
            return this.generateError('query was not of a recognised type', callback);
        var response = this.fireRequest(this.bridgeUrl, request, queryParams);
        if (callback) {
            response.then(function (res) { return callback(undefined, res); })
                .catch(function (err) { return callback(err); });
        }
        else {
            return response;
        }
    };
    AbstractBridge.prototype.extractRequest = function (query) {
        switch (typeof query) {
            case 'string': return [new query_1.Query(query).build(), {}];
            case 'object': return query instanceof query_1.Query ? [query.build(), query.queryParams] : [query, {}];
            default: return [null, null];
        }
    };
    AbstractBridge.prototype.generateError = function (error, callback) {
        var err = new Error(error);
        if (callback) {
            callback(err);
        }
        else {
            return Promise.reject(err);
        }
    };
    AbstractBridge.prototype.fireRequest = function (url, body, queryParams) {
        var _this = this;
        var options = {
            url: this.bridgeUrl,
            method: 'post',
            params: queryParams,
            data: this.augmentRequest(body),
            headers: this.headers,
            responseType: 'json',
            timeout: 1500
        };
        return axios(options)
            .then(function (res) { return res.data; })
            .then(function (res) { return res.records ? Object.assign(res, { records: res.records.map(_this.convertRecordFields) }) : res; });
    };
    AbstractBridge.prototype.convertRecordFields = function (record) {
        var converted = Object.assign(record, { id: record._id, url: record._u, title: record._t });
        delete converted._id;
        delete converted._u;
        delete converted._t;
        if (record._snippet) {
            converted.snippet = record._snippet;
            delete converted._snippet;
        }
        return converted;
    };
    return AbstractBridge;
}());
exports.AbstractBridge = AbstractBridge;
var CloudBridge = (function (_super) {
    __extends(CloudBridge, _super);
    function CloudBridge(clientKey, customerId) {
        _super.call(this);
        this.clientKey = clientKey;
        this.bridgeRefinementsUrl = null;
        this.bridgeRefinementsSearchUrl = null;
        this.bridgeClusterUrl = null;
        this.baseUrl = "https://" + customerId + ".groupbycloud.com:443/api/v1";
        this.bridgeUrl = this.baseUrl + SEARCH;
        this.bridgeRefinementsUrl = this.baseUrl + REFINEMENTS;
        this.bridgeRefinementsSearchUrl = this.baseUrl + REFINEMENT_SEARCH;
        this.bridgeClusterUrl = this.baseUrl + CLUSTER;
    }
    CloudBridge.prototype.augmentRequest = function (request) {
        return Object.assign(request, { clientKey: this.clientKey });
    };
    return CloudBridge;
}(AbstractBridge));
exports.CloudBridge = CloudBridge;
var BrowserBridge = (function (_super) {
    __extends(BrowserBridge, _super);
    function BrowserBridge(customerId, https) {
        if (https === void 0) { https = false; }
        _super.call(this);
        var scheme = https ? 'https' : 'http';
        var port = https ? ':443' : '';
        this.baseUrl = scheme + "://" + customerId + "-cors.groupbycloud.com" + port + "/api/v1";
        this.bridgeUrl = this.baseUrl + SEARCH;
    }
    BrowserBridge.prototype.augmentRequest = function (request) {
        return request;
    };
    return BrowserBridge;
}(AbstractBridge));
exports.BrowserBridge = BrowserBridge;
