import { Biasing, CustomUrlParam, MatchStrategy, Request, RestrictNavigation, SelectedRangeRefinement, SelectedValueRefinement, Sort } from '../models/request';
import { Navigation, RangeRefinement, ValueRefinement } from '../models/response';
export interface QueryConfiguration {
    userId?: string;
    language?: string;
    collection?: string;
    area?: string;
    biasingProfile?: string;
    pageSize?: number;
    fields?: string | string[];
}
export declare class Query {
    queryParams: any;
    private request;
    private unprocessedNavigations;
    constructor(query?: string);
    withQuery(query: string): Query;
    withConfiguration(configuration: QueryConfiguration, mask?: string): Query;
    withSelectedRefinements(...refinements: Array<SelectedValueRefinement | SelectedRangeRefinement>): Query;
    withoutSelectedRefinements(...refinements: Array<SelectedValueRefinement | SelectedRangeRefinement>): Query;
    withRefinements(navigationName: string, ...refinements: Array<ValueRefinement | RangeRefinement>): Query;
    withNavigations(...navigations: Navigation[]): Query;
    withCustomUrlParams(customUrlParams: CustomUrlParam[] | string): Query;
    withFields(...fields: string[]): Query;
    withOrFields(...orFields: string[]): Query;
    withSorts(...sorts: Sort[]): Query;
    withoutSorts(...sorts: Sort[]): Query;
    withIncludedNavigations(...navigationNames: string[]): Query;
    withExcludedNavigations(...navigationNames: string[]): Query;
    withQueryParams(queryParams: any | string): Query;
    refineByValue(navigationName: string, value: string, exclude?: boolean): Query;
    refineByRange(navigationName: string, low: number, high: number, exclude?: boolean): Query;
    restrictNavigation(restrictNavigation: RestrictNavigation): Query;
    skip(skip: number): Query;
    withPageSize(pageSize: number): Query;
    withMatchStrategy(matchStrategy: MatchStrategy): Query;
    withBiasing(biasing: Biasing): Query;
    enableWildcardSearch(): Query;
    disableAutocorrection(): Query;
    disableBinaryPayload(): Query;
    allowPrunedRefinements(): Query;
    build(): Request;
    readonly raw: Request;
    readonly rawNavigations: Navigation[];
    private addRefinement(refinement, refinements);
    private refinementMatches(target, original);
    private convertParamString(customUrlParams);
    private convertQueryString(queryParams);
    private clearEmptyArrays(request);
}
